# DeleteLinkedStacks.py (version 1.00)
# Script for deleting all figures in a multi-figure BookStack stack

# 1.00: (2024-09-03) Tested and working in Poser 6, Poser Pro 2014 and Poser 11.

import poser

# I put my scripts into a class simply to avoid possible conflict with stuff in the __main__ module.
class cheapskatesTemporaryClass(object):
	
	# For building a list of all the BookStack figures making up the current stack.
	figlist = []
	
	# Recursive function to get any BookStack figures to which this, or any of its parent, BookStack figures are parented
	def getAnyParentStackFigures(self,thisfig):
		act = thisfig.ParentActor()
		parentact = act.Parent()
		parentactname = parentact.Name()
		if parentactname[:10] == "TopOfStack":
			if parentact.ItsFigure():
				parentfig = parentact.ItsFigure()
				figname = parentfig.Name()
				if figname[:9] == "BookStack":
					self.figlist.append(figname)
					self.getAnyParentStackFigures(parentfig)
		return
	
	# Recursive function to get any BookStack figures that are parented to TopOfStack of the main, or of any nested, BookStack figure
	def getAnyChildStackFigure(self, thisfig):
		act = thisfig.ActorByInternalName("top_of_stack")
		actlist = act.Children()
		for a in actlist:
			if a.ItsFigure():
				childfig = a.ItsFigure()
				figname = childfig.Name()
				if figname[:9] == "BookStack":
					self.figlist.append(figname)
					self.getAnyChildStackFigure(childfig)
					
		return
	
	def main(self):
			
		# Abort if the selected figure's not a BookStack
		fig = poser.Scene().CurrentFigure()
		if not (fig.Name()[:9] == "BookStack"):
			poser.DialogSimple.MessageBox("This script is only for use with the 3DCheapskate's 'BookStack' figures.\n\nScript aborted, no changes made.")
			return
		
		# Get parent stacks recursively, and then reverse the list so that the ultimate parent of the whole stack is first
		self.getAnyParentStackFigures(fig)
		self.figlist.reverse()
		
		# ...then add the current figure to the list
		self.figlist.append(fig.Name())
		
		# ...then get child stacks recursively
		self.getAnyChildStackFigure(fig)
		
		# Final confirmation
		confirm1 = poser.DialogSimple.YesNo("Delete the following figure and all linked BookStack figures ?\n\n" + fig.Name() + "\n\n(final confirmation)")
		if confirm1:
			
			for f in self.figlist:
				fig = poser.Scene().Figure(f)
				poser.Scene().SelectFigure(fig)
				poser.Scene().DeleteCurrentFigure()
				
			# Redraw what we've modified	
			poser.Scene().Draw()
			
#poser.DialogSimple.MessageBox("Running 3DCheapskates DeleteLinkedStacks.py (version 1.00) script\n\n(N.B. You should see a similar message when the script finishes. If you don't then the script has crashed!)")
cheapskatesTemporaryInstance = cheapskatesTemporaryClass()
cheapskatesTemporaryInstance.main()
#poser.DialogSimple.MessageBox("3DCheapskates DeleteLinkedStacks.py (version 1.00) script all done.")