# MatPulp02.py (version 1.00)
# Script for loading a material collection to all figures in a multi-figure BookStack stack.

# 1.00: (2024-09-03) Tested and working in Poser 6, Poser Pro 2014 and Poser 11.

import poser

# I put my scripts into a class simply to avoid possible conflict with stuff in the __main__ module.
class cheapskatesTemporaryClass(object):
	
	# For building a list of all the BookStack figures making up the current stack.
	figlist = []
	
	# Recursive function to get any BookStack figures to which this, or any of its parent, BookStack figures are parented
	def getAnyParentStackFigures(self,thisfig):
		act = thisfig.ParentActor()
		parentact = act.Parent()
		parentactname = parentact.Name()
		if parentactname[:10] == "TopOfStack":
			if parentact.ItsFigure():
				parentfig = parentact.ItsFigure()
				figname = parentfig.Name()
				if figname[:9] == "BookStack":
					self.figlist.append(figname)
					self.getAnyParentStackFigures(parentfig)
		return
	
	# Recursive function to get any BookStack figures that are parented to TopOfStack of the main, or of any nested, BookStack figure
	def getAnyChildStackFigure(self, thisfig):
		act = thisfig.ActorByInternalName("top_of_stack")
		actlist = act.Children()
		for a in actlist:
			if a.ItsFigure():
				childfig = a.ItsFigure()
				figname = childfig.Name()
				if figname[:9] == "BookStack":
					self.figlist.append(figname)
					self.getAnyChildStackFigure(childfig)
					
		return
	
	def main(self):
			
		# Abort if the selected figure's not a BookStack
		fig = poser.Scene().CurrentFigure()
		if not (fig.Name()[:9] == "BookStack"):
			poser.DialogSimple.MessageBox("This script is only for use with the 3DCheapskate's 'BookStack' figures.\n\nScript aborted, no changes made.")
			return
		
		# Get parent stacks recursively, and then reverse the list so that the ultimate parent of the whole stack is first
		self.getAnyParentStackFigures(fig)
		self.figlist.reverse()
		
		# ...then add the current figure to the list
		self.figlist.append(fig.Name())
		
		# ...then get child stacks recursively
		self.getAnyChildStackFigure(fig)
			
		cnt = 0 # count is 0 to 63
		col = 0 # column is either 0 or 1
		book = 0 # book is 0 to 31
		bkthisfig = 0 # number of books in this figure (obtained from figure name)
		for f in self.figlist:
			fig = poser.Scene().Figure(f)
			act = fig.Actor("Body")
			act.LoadMaterialCollection(":Runtime:Libraries:Materials:3DCheapskate:TEST:Bookstack4:Pulp02.mc6")
			
			# Extract the number of books in the figure from the name, e.g. "BookStack4 - books 1-4", "BookStack8 - books 1-8", "BookStack32 - books 1-32"
			# (This is in case I ever create the larger BookStack figures)
			if fig.Name()[9].isdigit():
				if fig.Name()[10].isdigit():
					bkthisfig = int(fig.Name()[9])*10 + int(fig.Name()[10])
				else:
					bkthisfig = int(fig.Name()[9])
			
			# Determine COLUMN_0_OR_1 and BOOK_0_TO_31 settings for this figure
			col = 0
			book = cnt
			if cnt > 31:
				book = cnt - 32
				col = 1
			#print (fig.Name() + ": cnt= " + str(cnt) + ",bkthisfig = " + str(bkthisfig) +", col="+str(col)+", book="+str(book))
			
			# Set the book number and column for each figure appropriately in the cover material...
			st = fig.Material("BookMap64Cover").ShaderTree()
			nodelist = st.Nodes()
			for n in nodelist:
				if n.Name() == "COLUMN_0_OR_1":
					n.InputByInternalName("Value_1").SetFloat(col)
				if n.Name() == "BOOK_0_TO_31":
					n.InputByInternalName("Value_1").SetFloat(book)
			
			# Set the book number and column for each figure appropriately in the cover material...
			st = fig.Material("BookMap64Pages").ShaderTree()
			nodelist = st.Nodes()
			for n in nodelist:
				if n.Name() == "COLUMN_0_OR_1":
					n.InputByInternalName("Value_1").SetFloat(col)
				if n.Name() == "BOOK_0_TO_31":
					n.InputByInternalName("Value_1").SetFloat(book)
					
			# Increase book counter by the number of books in this figure, keeping the result modulo 64
			cnt = cnt + bkthisfig
			if cnt >63:
				cnt = cnt - 64
			
		# Redraw what we've modified	
		poser.Scene().Draw()
			
		
#poser.DialogSimple.MessageBox("Running 3DCheapskates MatPulp02.py (version 1.00) script\n\n(N.B. You should see a similar message when the script finishes. If you don't then the script has crashed!)")
cheapskatesTemporaryInstance = cheapskatesTemporaryClass()
cheapskatesTemporaryInstance.main()
#poser.DialogSimple.MessageBox("3DCheapskates MatPulp02.py (version 1.00) script all done.")