# RandomizeBookStack.py (version 1.00)
# Script for randomizing stacks of my BookStack4 figures

# 0.03: Added option to select subsets of alternate geometries

# 1.00: (2024-09-03) Tested and working in Poser 6, Poser Pro 2014 and Poser 11.


import poser,random

# I put my scripts into a class simply to avoid possible conflict with stuff in the __main__ module.
class cheapskatesTemporaryClass(object):
	
	# List of all the BookStack4 figures linked underneath the selected one
	figlist = []
	
	# Recursive function to get any BookStack4 figures to which this, or any of its parent, BookStack4 figures are parented
	def getAnyParentStackFigures(self,thisfig):
		act = thisfig.ParentActor()
		parentact = act.Parent()
		parentactname = parentact.Name()
		if parentactname[:10] == "TopOfStack":
			if parentact.ItsFigure():
				parentfig = parentact.ItsFigure()
				figname = parentfig.Name()
				if figname[:9] == "BookStack":
					self.figlist.append(figname)
					self.getAnyParentStackFigures(parentfig)
		return
	
	# Recursive function to get any BookStack4 figures that are parented to TopOfStack of the main, or of any nested, BookStack4 figure
	def getAnyChildStackFigures(self, thisfig):
		act = thisfig.ActorByInternalName("top_of_stack")
		actlist = act.Children()
		for a in actlist:
			if a.ItsFigure():
				childfig = a.ItsFigure()
				figname = childfig.Name()
				if figname[:9] == "BookStack":
					self.figlist.append(figname)
					self.getAnyChildStackFigures(childfig)
					
		return
	
	# Popup dialogue to enter an integer
	def enterInteger(self):
		txt = poser.DialogTextEntry(message="Rotation limit in degrees:")
		txt.Show()
		if txt.Text().isdigit():
			return int(txt.Text())
		else:
			poser.DialogSimple.MessageBox("Invalid input - using 10")
			return 10
	
	# Popup dialogue to enter an integer
	def enterFloat(self):
		txt = poser.DialogTextEntry(message="Rotation limit in degrees:")
		txt.Show()
		s = txt.Text()
		if s.replace(".", "").isnumeric():
			return float(txt.Text())
		else:
			poser.DialogSimple.MessageBox("Invalid input - using 1.0")
			return 1.0
	
	def main(self):
		
		# Get Poser version for doing version-specific stuff
		poserversion = '0'
		posermainversion = '0'
		try:
			self.poserversion = poser.AppVersion()
			temp = self.poserversion.split('.')
			self.posermainversion = temp[0]
		except:
			pass
			
		# Abort if the user gets cold feet right at the start
		confirmed = poser.DialogSimple.YesNo("This script randomizes the books in the stack.\n\nDo you wish to continue?")
		if not confirmed:
			confirmed = poser.DialogSimple.YesNo("Do you want to see more information about what this script does?\n\n(This opens a debug window with extra information and terminates the script. You'll probably need to resize the debug window to read the information.)")
			if confirmed:
				# Briefly explain to the user what this script does and give an option toabort straight away
				print "This script randomizes the books in the stack... MORE EXPLANATION TO GO HERE"
			return
			
		# Abort if the selected figure's not a BookStack
		fig = poser.Scene().CurrentFigure()
		if not (fig.Name()[:9] == "BookStack"):
			poser.DialogSimple.MessageBox("This script is only for use with the 3DCheapskate's 'BookStack' figures.\n\nScript aborted, no changes made.")
			return
		
		# Get parent stacks recursively, and then reverse the list so that the ultimate parent of the whole stack is first
		parentstoo=poser.DialogSimple.YesNo("Parent bookstack figures too?")
		if parentstoo:
			self.getAnyParentStackFigures(fig)
			self.figlist.reverse()
		
		# ...then add the current figure to the list
		self.figlist.append(fig.Name())
		
		# ...then get child stacks recursively
		kidstoo=poser.DialogSimple.YesNo("Child bookstack figures too?")
		if kidstoo:
			self.getAnyChildStackFigures(fig)
		
		# Randomize geometries ?
		geomtoo = 0
		geoms = []
		# Ask user which geometries
		geomMenu = ("Any","Hardback","Hardback (curved spine)","Hardback (flat spine)","Paperback","Any, but right way up","Hardback, right way up","Hardback (curved spine), right way up","Hardback (flat spine), right way up","Paperback, right way up","Any, upside down","Hardback, upside down","Hardback (curved spine), upside down","Hardback (flat spine), upside down","Paperback, upside down")
		geomVals = ((0,1,2,3,4,5),(0,1,2,3),(0,1),(2,3),(4,5),(0,2,4),(0,2),(0,0),(2,2),(4,4),(1,3,5),(1,3),(1,1),(3,3),(5,5))
		# N.B. The 2-tuples with duplicate numbers are because if geoms has a single number in it then 'len(geoms)' errors. Duplicating the number is a workaround
		sel = poser.DialogSimple.AskMenu('Randomize geometries','Randomization:',geomMenu)
		if sel:
			geomtoo = 1
			i=0
			for g in geomMenu:
				if sel == g:
					geoms = geomVals[i]
				i = i + 1
		
		# Randomize morphs+adjustments ?
		adjtoo=poser.DialogSimple.YesNo("Randomize morphs + adjustments ?")
		
		# Randomize yrot ?
		yrottoo=0
		yrotopt = 0 # 0=no yrot, 1=-180 to 180, 2=-10 to 10, 3=(-10 to 10) or (170 to 190) ...N.B. the +-10 is a value the user can select
		# Ask user what rotations
		yrotmenu = ("Full 360 degree rotation","Small rotation from 0", "Small rotation from 0 or 180")
		yrotlim = 0
		sel = poser.DialogSimple.AskMenu('Randomize y rotations','Randomization:',yrotmenu)
		if sel:
			yrottoo = 1
			i=0
			for r in yrotmenu:
				if sel == r:
					yrotopt=i+1
				i = i + 1
		else:
			yrottoo=0 # if user selects default 'none' option
		if (yrotopt == 2) or (yrotopt == 3):
			yrotlim = self.enterInteger()
		
		# Randomize translations ?
		transtoo=poser.DialogSimple.YesNo("Randomize positions ?")
		
		# Randomize scaling ?
		scalestoo=poser.DialogSimple.YesNo("Randomize book dimensions ?")
		
		# Final confirmation
		confirm1=poser.DialogSimple.YesNo("Ready to randomize?\n(final confirmation)")
		if confirm1:
			
			bookcount = 0.0
			bkthisfig = 0 # number of books in this figure (obtained from figure name)
			for f in self.figlist:
				fig = poser.Scene().Figure(f)
			
				# Extract the number of books in the figure from the name, e.g. "BookStack4 - books 1-4", "BookStack8 - books 1-8", "BookStack32 - books 1-32"
				# (This is in case I ever create the larger BookStack figures)
				if fig.Name()[9].isdigit():
					if fig.Name()[10].isdigit():
						bkthisfig = int(fig.Name()[9])*10 + int(fig.Name()[10])
					else:
						bkthisfig = int(fig.Name()[9])
				
				st = fig.Material("BookMap64Cover").ShaderTree()
				nodelist = st.Nodes()
				for n in nodelist:
					if n.Name() == "BOOK_0_TO_31":
						n.InputByInternalName("Value_1").SetFloat(bookcount)
						bookcount = bookcount + bkthisfig
				# For each of the four books in the figure...
				i = 1
				while i < (bkthisfig+1):
					# Get the next book
					bone = "book_" + str(i)
					act = fig.ActorByInternalName(bone)
				
					# First reset all the standard transforms (although hidden they can still be twiddled with the editing tools, translate, rotate, scale and twist)
					p = act.Parameter("xRotate").SetValue(0)
					p = act.Parameter("yRotate").SetValue(0)
					p = act.Parameter("zRotate").SetValue(0)
					p = act.Parameter("Scale").SetValue(1)
					p = act.Parameter("xScale").SetValue(1)
					p = act.Parameter("yScale").SetValue(1)
					p = act.Parameter("zScale").SetValue(1)
					p = act.Parameter("xTran").SetValue(0)
					p = act.Parameter("yTran").SetValue(0)
					p = act.Parameter("yTran").SetValue(0)
									
					# Randomize geometry
					if geomtoo:
						pname = "Book" + str(i)+"Geom"
						p = act.Parameter(pname)
						rndcnt = len(geoms)
						rand = random.randint(0,rndcnt-1)
						rand = geoms[rand]
						#rand = random.randint(p.MinValue(),p.MaxValue()) #int, can be negative
						p.SetValue(rand)
					
					if adjtoo:
						# Randomize 'Splay'
						p = act.Parameter("Splay")
						rand = random.uniform(p.MinValue(),p.MaxValue())
						p.SetValue(rand)
						
						# Randomize 'Oblique'
						p = act.Parameter("Oblique")
						rand = random.uniform(p.MinValue(),p.MaxValue())
						p.SetValue(rand)
						
						# Randomize 'Shear'
						p = act.Parameter("Shear")
						rand = random.uniform(p.MinValue(),p.MaxValue())
						p.SetValue(rand)
						
						# Randomize 'Plump' - OVERRIDE -1 MINIMUM VALUE, USE 0
						p = act.Parameter("Plump")
						rand = random.uniform(0,p.MaxValue()) #float, can be negative
						p.SetValue(rand)
						
						# Randomize 'Shear'
						p = act.Parameter("Shear")
						rand = random.uniform(p.MinValue(),p.MaxValue())
						p.SetValue(rand)
						
						# Randomize 'Twist'
						p = act.Parameter("Twist")
						rand = random.uniform(p.MinValue(),p.MaxValue())
						p.SetValue(rand)
						
						# Randomize 'Curl (Paperback only)' - BEST NOT TO DO THIS VIA SCRIPT
						#p = act.Parameter("Curl (Paperback only)")
						#rand = random.uniform(p.MinValue(),p.MaxValue())
						#p.SetValue(rand)
					
					# Randomize rotation
					# yrotopt : 0=no yrot, 1=-180 to 180, 2=-10 to 10, 3=(-10 to 10) or (170 to 190) ...N.B. the +-10 is a value the user can select
					if yrottoo:
						pmin=0
						pmax=0
						if yrotopt == 1:
							pmin=-180
							pmax=180
						else:
							if yrotopt == 2:
								pmin=-yrotlim
								pmax=yrotlim
							if yrotopt == 3:
								# randomly select whether -10 to +10 or 170 to 190
								pmin=-yrotlim
								pmax=yrotlim
								rand = random.uniform(-1,1)
								if rand > 0:
									pmin=180-yrotlim
									pmax=180+yrotlim
						p = act.Parameter("Rotate")
						rand = random.uniform(pmin,pmax)
						p.SetValue(rand)
					
					if transtoo:
						# Randomize 'xTran'
						pmin=0
						pmax=0
						p = act.Parameter("Nudge sideways")
						rand = random.uniform(p.MinValue(),p.MaxValue())
						p.SetValue(rand)
						
						# Randomize 'zTran'
						pmin=0
						pmax=0
						p = act.Parameter("Nudge lengthwise")
						rand = random.uniform(p.MinValue(),p.MaxValue())
						p.SetValue(rand)
					
					if scalestoo:
						# Randomize 'Cover proportions'
						pmin=0
						pmax=0
						p = act.Parameter("Cover proportions")
						rand = random.uniform(p.MinValue(),p.MaxValue())
						p.SetValue(rand)
						
						# Randomize 'Scale'
						pmin=0
						pmax=0
						p = act.Parameter("Book size")
						rand = random.uniform(p.MinValue(),p.MaxValue())
						p.SetValue(rand)
						
						# Randomize 'yScale'
						pmin=0
						pmax=0
						p = act.Parameter("Book thickness")
						rand = random.uniform(p.MinValue(),p.MaxValue())
						p.SetValue(rand)
					
					# Next book if any
					i = i + 1
				
			# Redraw what we've modified	
			poser.Scene().Draw()
			
			# If the geometry is changed then Poser 11 unhides all the morphs for all geometries, so re-hide them
			for f in self.figlist:
				fig = poser.Scene().Figure(f)
			
				# Extract the number of books in the figure from the name, e.g. "BookStack4 - books 1-4", "BookStack8 - books 1-8", "BookStack32 - books 1-32"
				# (This is in case I ever create the larger BookStack figures)
				if fig.Name()[9].isdigit():
					if fig.Name()[10].isdigit():
						bkthisfig = int(fig.Name()[9])*10 + int(fig.Name()[10])
					else:
						bkthisfig = int(fig.Name()[9])
				
				# For each of the books in the figure...
				i = 1
				while i < (bkthisfig+1):
					# Get the next book
					bone = "book_" + str(i)
					act = fig.ActorByInternalName(bone)
					nodename = "MT HBc" + str(i) + " Plump"
					act.Parameter(nodename).SetHidden(1)
					nodename = "MT HBc" + str(i) + " Shear"
					act.Parameter(nodename).SetHidden(1)
					nodename = "MT HBc" + str(i) + " Twist"
					act.Parameter(nodename).SetHidden(1)
					nodename = "MT HBf" + str(i) + " Plump"
					act.Parameter(nodename).SetHidden(1)
					nodename = "MT HBf" + str(i) + " Shear"
					act.Parameter(nodename).SetHidden(1)
					nodename = "MT HBf" + str(i) + " Twist"
					act.Parameter(nodename).SetHidden(1)
					nodename = "MT PB" + str(i) + " Curl"
					act.Parameter(nodename).SetHidden(1)
					nodename = "MT PB" + str(i) + " Plump"
					act.Parameter(nodename).SetHidden(1)
					nodename = "MT PB" + str(i) + " Shear"
					act.Parameter(nodename).SetHidden(1)
					nodename = "MT PB" + str(i) + " Twist"
					act.Parameter(nodename).SetHidden(1)
					
					i = i + 1
				
			# Redraw what we've modified	
			poser.Scene().Draw()
			
			# Inform the user that it's all done
			poser.DialogSimple.MessageBox("All done")
			
poser.DialogSimple.MessageBox("Running 3DCheapskates RandomizeBookStack.py (version 1.00) script\n\n(N.B. You should see a similar message when the script finishes. If you don't then the script has crashed!)")
cheapskatesTemporaryInstance = cheapskatesTemporaryClass()
cheapskatesTemporaryInstance.main()
poser.DialogSimple.MessageBox("3DCheapskates RandomizeBookStack.py (version 1.00) script all done.")