# ResetBookStack4.py (version 1.00)
# Script for resetting stacks of my BookStack4 figures
# This is basically ResetBookStack4.py (version 1.00) with all dialogs and prints removed

# 1.00: (2024-09-03) Tested and working in Poser 6, Poser Pro 2014 and Poser 11.


import poser,random

# I put my scripts into a class simply to avoid possible conflict with stuff in the __main__ module.
class cheapskatesTemporaryClass(object):
	
	# List of all the BookStack4 figures linked underneath the selected one
	figlist = []
	
	# Recursive function to get any BookStack4 figures to which this, or any of its parent, BookStack4 figures are parented
	def getAnyParentStackFigures(self,thisfig):
		act = thisfig.ParentActor()
		parentact = act.Parent()
		parentactname = parentact.Name()
		if parentactname[:10] == "TopOfStack":
			if parentact.ItsFigure():
				parentfig = parentact.ItsFigure()
				figname = parentfig.Name()
				if figname[:9] == "BookStack":
					self.figlist.append(figname)
					self.getAnyParentStackFigures(parentfig)
		return
	
	# Recursive function to get any BookStack4 figures that are parented to TopOfStack of the main, or of any nested, BookStack4 figure
	def getAnyChildStackFigures(self, thisfig):
		
		act = thisfig.ActorByInternalName("top_of_stack")
		actlist = act.Children()
		for a in actlist:
			if a.ItsFigure():
				childfig = a.ItsFigure()
				figname = childfig.Name()
				if figname[:9] == "BookStack":
					self.figlist.append(figname)
					self.getAnyChildStackFigures(childfig)
					
		return
	
	def main(self):
		
		# Get Poser version for doing version-specific stuff
		poserversion = '0'
		posermainversion = '0'
		try:
			self.poserversion = poser.AppVersion()
			temp = self.poserversion.split('.')
			self.posermainversion = temp[0]
		except:
			pass
			
		# Abort if the selected figure's not a BookStack4
		fig = poser.Scene().CurrentFigure()
		if not (fig.Name()[:9] == "BookStack"):
			poser.DialogSimple.MessageBox("This script is only for use with the 3DCheapskate's 'BookStack' figures.\n\nScript aborted, no changes made.")
			return
		
		# Get parent stacks recursively, and then reverse the list so that the ultimate parent of the whole stack is first
		self.getAnyParentStackFigures(fig)
		self.figlist.reverse()
		
		# ...then add the current figure to the list
		self.figlist.append(fig.Name())
		
		# ...then get child stacks recursively
		self.getAnyChildStackFigures(fig)
		
		bkthisfig = 0 # number of books in this figure (obtained from figure name)	
		for f in self.figlist:
		
			# Get the figure
			fig = poser.Scene().Figure(f)
			
			# Extract the number of books in the figure from the name, e.g. "BookStack4 - books 1-4", "BookStack8 - books 1-8", "BookStack32 - books 1-32"
			# (This is in case I ever create the larger BookStack figures)
			if fig.Name()[9].isdigit():
				if fig.Name()[10].isdigit():
					bkthisfig = int(fig.Name()[9])*10 + int(fig.Name()[10])
				else:
					bkthisfig = int(fig.Name()[9])
			
			# DON'T TOUCH THE MATERIALS WHEN RESETTING
					
			# For each of the books in the figure
			i = 1
			while i < (bkthisfig+1):
				
				# Get the relevant bone/bodypart
				bone = "book_" + str(i)
				act = fig.ActorByInternalName(bone)
				
				# First reset all the standard transforms (although hidden they can still be twiddled with the editing tools, translate, rotate, scale and twist)
				p = act.Parameter("xRotate").SetValue(0)
				p = act.Parameter("yRotate").SetValue(0)
				p = act.Parameter("zRotate").SetValue(0)
				p = act.Parameter("Scale").SetValue(1)
				p = act.Parameter("xScale").SetValue(1)
				p = act.Parameter("yScale").SetValue(1)
				p = act.Parameter("zScale").SetValue(1)
				p = act.Parameter("xTran").SetValue(0)
				p = act.Parameter("yTran").SetValue(0)
				p = act.Parameter("yTran").SetValue(0)
								
				# Reset geometry
				pname = "Book" + str(i)+"Geom"
				p = act.Parameter(pname)
				p.SetValue(0.0)
				
				# Reset morphs + adjustments
				p = act.Parameter("Splay")
				p.SetValue(0)
				
				p = act.Parameter("Oblique")
				p.SetValue(0)
				
				p = act.Parameter("Shear")
				p.SetValue(0)
				
				p = act.Parameter("Plump")
				p.SetValue(0)
				
				p = act.Parameter("Shear")
				p.SetValue(0)
				
				p = act.Parameter("Twist")
				p.SetValue(0)
				
				p = act.Parameter("Curl (Paperback only)")
				p.SetValue(0)
				
				# Reset Y rotation
				p = act.Parameter("Rotate")
				p.SetValue(0)
				
				# Reset position
				p = act.Parameter("Nudge sideways")
				p.SetValue(0)
				
				p = act.Parameter("Nudge lengthwise")
				p.SetValue(0)
				
				# Reset dimensions
				p = act.Parameter("Cover proportions")
				p.SetValue(1)
				
				p = act.Parameter("Book size")
				p.SetValue(1)
				
				p = act.Parameter("Book thickness")
				p.SetValue(1)
				
				# Increment book number for next iteration
				i = i + 1
			
		# Redraw what we've modified	
		poser.Scene().Draw()
		
		# If the geometry is changed then Poser11 unhides all the morphs for all geometries, so I need to re-hide them
		for f in self.figlist:
			fig = poser.Scene().Figure(f)
			
			# Extract the number of books in the figure from the name, e.g. "BookStack4 - books 1-4", "BookStack8 - books 1-8", "BookStack32 - books 1-32"
			# (This is in case I ever create the larger BookStack figures)
			if fig.Name()[9].isdigit():
				if fig.Name()[10].isdigit():
					bkthisfig = int(fig.Name()[9])*10 + int(fig.Name()[10])
				else:
					bkthisfig = int(fig.Name()[9])
			
			# For each of the books in the figure...
			i = 1
			while i < (bkthisfig+1):
				# Get the next book
				bone = "book_" + str(i)
				
				# Hide all the relevant parameter dials
				act = fig.ActorByInternalName(bone)
				nodename = "MT HBc" + str(i) + " Plump"
				act.Parameter(nodename).SetHidden(1)
				nodename = "MT HBc" + str(i) + " Shear"
				act.Parameter(nodename).SetHidden(1)
				nodename = "MT HBc" + str(i) + " Twist"
				act.Parameter(nodename).SetHidden(1)
				nodename = "MT HBf" + str(i) + " Plump"
				act.Parameter(nodename).SetHidden(1)
				nodename = "MT HBf" + str(i) + " Shear"
				act.Parameter(nodename).SetHidden(1)
				nodename = "MT HBf" + str(i) + " Twist"
				act.Parameter(nodename).SetHidden(1)
				nodename = "MT PB" + str(i) + " Curl"
				act.Parameter(nodename).SetHidden(1)
				nodename = "MT PB" + str(i) + " Plump"
				act.Parameter(nodename).SetHidden(1)
				nodename = "MT PB" + str(i) + " Shear"
				act.Parameter(nodename).SetHidden(1)
				nodename = "MT PB" + str(i) + " Twist"
				act.Parameter(nodename).SetHidden(1)
				
				i = i + 1
			
cheapskatesTemporaryInstance = cheapskatesTemporaryClass()
cheapskatesTemporaryInstance.main()
